/* Open SoundControl kit in C++                                              */
/* Copyright (C) 2002 Daniel Holth, Clinton McChesney                        */
/*                                                                           */
/* This library is free software; you can redistribute it and/or             */
/* modify it under the terms of the GNU Lesser General Public                */
/* License as published by the Free Software Foundation; either              */
/* version 2.1 of the License, or (at your option) any later version.        */
/*                                                                           */
/* This library is distributed in the hope that it will be useful,           */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of            */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         */
/* Lesser General Public License for more details.                           */
/*                                                                           */
/* You should have received a copy of the GNU Lesser General Public          */
/* License along with this library; if not, write to the Free Software       */
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
/*                                                                           */
/* For questions regarding this program contact                              */
/* Daniel Holth <dholth@stetson.edu> or visit                                */
/* http://www.stetson.edu/~ProctoLogic/                                      */

/**
 * @class MasterBroadcaster
 *
 * Uses "Round Robin" method of scheduling to transmit queued messages.
 *
 * @warning Assumes that the broadcaster namespace is static.
 *
 * @author Daniel Holth
 * @author Clinton McChesney
 */
 
#ifndef MASTER_BROADCASTER_H
#define MASTER_BROADCASTER_H

#include <iostream>
#include <semaphore.h>
#include <pthread.h>
#include "BroadcasterNamespace.h"
#include "QueuedBroadcaster.h"


class MasterBroadcaster
{
  private:
    BroadcasterNamespace* myBroadcasters;
    sem_t                 mySemaphore;
    pthread_t             myThread;
    
  protected:
    
  public:
    MasterBroadcaster();
    virtual ~MasterBroadcaster();

    BroadcasterNamespace* getBroadcasters()       { return this->myBroadcasters; }
    void setBroadcasters(BroadcasterNamespace* b) { this->myBroadcasters = b; }

    /// Create the broadcasting thread.  Returns immediately.
    void startThread();

    /// The broadcasting thread.
    void run();

    sem_t* getSemaphore() { return &this->mySemaphore; }
    pthread_t* getThread(){ return &this->myThread;    }
};

#endif
