/**
 * @class
 *
 * Replies to a documentation request.
 *
 * Sends a documentation reply with the docstring
 * of the queried object to the requester of the
 * documentation.
 */

#ifndef OSCDOCUMENTOR_H
#define OSCDOCUMENTOR_H

#include "OSCPacker.h"
#include "OSCUnpacker.h"
#include "OSCNamespace.h"
#include "OSCCallable.h"


class OSCDocumentor : public OSCCallable
{
  private:
    OSCNamespace *mySpace;
  protected:
    OSCNamespace* getNamespace()      { return this->mySpace; }
  public:
    OSCDocumentor()          { }
    virtual ~OSCDocumentor() { }

    void setNamespace(OSCNamespace *space)  { this->mySpace = space; }
    
    void call(const std::string& data, Transmit *const replier) 
    {
      std::string queryAddress;
      std::string docstring;
      std::string reply;
      OSCNamespace  *space;
      OSCCallable   *queryItem;
      OSCUnpacker u;
      OSCPacker   p;

      u.setData(data);
      u.unpackString(&queryAddress);
      u.unpackString(&queryAddress);
      u.unpackString(&queryAddress);

      cout << "Doc query on " << queryAddress << endl;
      space     = this->getNamespace();
      queryItem = space->find(queryAddress);

      if(queryItem != NULL)
      {
        docstring = queryItem->getDocumentation();
        p.packString("/documented");
        p.packString(",s");
        p.packString(docstring.c_str());
      }
      else
      {
        p.packString("/documented");
        p.packString(",s");
        p.packString("item not found");
      }

      reply = p.getData();
      replier->transmit(reply);
    };
};

#endif
