/* Open SoundControl kit in C++                                              */
/* Copyright (C) 2002 Daniel Holth, Clinton McChesney                        */
/*                                                                           */
/* This library is free software; you can redistribute it and/or             */
/* modify it under the terms of the GNU Lesser General Public                */
/* License as published by the Free Software Foundation; either              */
/* version 2.1 of the License, or (at your option) any later version.        */
/*                                                                           */
/* This library is distributed in the hope that it will be useful,           */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of            */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         */
/* Lesser General Public License for more details.                           */
/*                                                                           */
/* You should have received a copy of the GNU Lesser General Public          */
/* License along with this library; if not, write to the Free Software       */
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
/*                                                                           */
/* For questions regarding this program contact                              */
/* Daniel Holth <dholth@stetson.edu> or visit                                */
/* http://www.stetson.edu/~ProctoLogic/                                      */

/**
 * @class OSCNamespace
 *
 * This class is responsible for finding the OSCCallable object that
 * corresponds to a particular address.
 *
 * Typical usage:
 * @code
 * void process(OSCNamespace *namespace, string *address)
 * {
 *   OSCCallable *callable;
 *   callable = namespace->find(*address);
 *   if(callable != NULL)
 *   {
 *     // We have found the callback for the given address.
 *   }
 * }
 * @endcode
 * 
 * @author Daniel Holth
 * @author Clinton McChesney
 */

#ifndef OSCNAMESPACE_H
#define OSCNAMESPACE_H

#include <string>
#include <vector>
#include "OSCCallable.h"

class OSCNamespace
{
  private:
  protected:
  public:
    virtual ~OSCNamespace(void) = 0;
    
    /// Add OSCCallable @a thingamabob to the namespace 
    /// with address @a address.
    virtual bool add(const std::string &address, OSCCallable *thingamabob) = 0;

    /// Remove the callable at address @a address.
    virtual bool remove(const std::string &address) { return false; }
    
    /// Get a pointer to the OSCCallable with the given address, or NULL
    /// if not found.
    virtual OSCCallable* find(const std::string& address) = 0;
   
    /// Return a list of all addresses in the namespace beginning with
    /// the string @a base.
    virtual std::vector<std::string> list(const std::string& base)  = 0;
};

#endif
