/**
 * @class OSCTypeReturner
 *
 * Returns the type signature
 *
 * Sends a reply with the type signature
 * of the queried object as well as the 
 * address of the queried object to the 
 * requester of the type signature.
 */

#ifndef OSCTYPE_RETURNER_H
#define OSCTYPE_RETURNER_H

#include "OSCPacker.h"
#include "OSCUnpacker.h"
#include "OSCNamespace.h"
#include "OSCCallable.h"

class OSCTypeReturner : public OSCCallable
{
  private:
    OSCNamespace *mySpace;
  protected:
    OSCNamespace* getNamespace()      { return this->mySpace; }
  public:
    OSCTypeReturner()          { }
    virtual ~OSCTypeReturner() { }

    void setNamespace(OSCNamespace *space)  { this->mySpace = space; }
    
    void call(const std::string& data, Transmit *const replier) 
    {
      std::string queryAddress;
      std::string typeSignature;
      std::string reply;
      OSCNamespace  *space;
      OSCCallable   *queryItem;
      OSCUnpacker u;
      OSCPacker   p;

      u.setData(data);
      u.unpackString(&queryAddress);
      u.unpackString(&queryAddress);
      u.unpackString(&queryAddress);

      cout << "Type Signature request on: " << queryAddress << endl;  // DEBUG
      space     = this->getNamespace();
      queryItem = space->find(queryAddress);

      if(queryItem != NULL)
      {
        typeSignature = queryItem->getTypeSignature();
        p.packString("/typeSig");
        p.packString(",ss");
        p.packString(queryAddress.c_str());
        p.packString(typeSignature.c_str());
      }
      else
      {
        // Is this the best approach for an error? (may confuse the recipient)
        p.packString("/typeSig");
        p.packString(",s");
        p.packString("item not found");
      }

      reply = p.getData();
      replier->transmit(reply);
    };
};

#endif
