/* Open SoundControl kit in C++                                              */
/* Copyright (C) 2002 Daniel Holth, Clinton McChesney                        */
/*                                                                           */
/* This library is free software; you can redistribute it and/or             */
/* modify it under the terms of the GNU Lesser General Public                */
/* License as published by the Free Software Foundation; either              */
/* version 2.1 of the License, or (at your option) any later version.        */
/*                                                                           */
/* This library is distributed in the hope that it will be useful,           */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of            */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         */
/* Lesser General Public License for more details.                           */
/*                                                                           */
/* You should have received a copy of the GNU Lesser General Public          */
/* License along with this library; if not, write to the Free Software       */
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
/*                                                                           */
/* For questions regarding this program contact                              */
/* Daniel Holth <dholth@stetson.edu> or visit                                */
/* http://www.stetson.edu/~ProctoLogic/                                      */

/* this class was modified by Martin Rumori (martin@rumori.de) */

/**
 * @file OSCProcessor.cc
 *
 * Implementation.
 * Accepts OSC Packets and sends them to the Namespace.
 *
 * @author Daniel Holth
 * @author Clinton McChesney
 * @author Martin Rumori
 */

#include "OSCProcessor.h"

using namespace std;

OSCProcessor::OSCProcessor ()
{

}


OSCProcessor::~OSCProcessor ()
{

}


bool OSCProcessor::process(const std::string& s, Transmit *const reply)
{
  OSCUnpacker unpacker;
  OSCCallable* callable;
  std::string address;
  
  unpacker.setData(s); // init unpacker
  unpacker.unpackString(&address); // get address

  // check for bundle
  if (address == "#bundle")
    {

#ifdef LIBOSC_DEBUG
      cerr << "OSCProcessor::process(): processing bundle" << endl;
#endif

      unpacker.skipHyper(); // skip timetag

      std::string bundleElem;

      while (unpacker.unpackBlob(&bundleElem))
      {
	// recursively process bundleElem
	process(bundleElem, reply);
      }
    }
  else
    { // no bundle, but message

#ifdef LIBOSC_DEBUG
      cerr << "OSCProcessor::process(): Processing address " << address << endl;
#endif

      // now get our callable object from the namespace
      if (myNamespace)
	{
	  callable = myNamespace->find(address);
	  if (callable)
	    {
#ifdef LIBOSC_DEBUG
		  cout << "callable for " << address << " is " << (*callable) << endl;
#endif
	      // and finally call the callable
	      callable -> call(s, reply);
	    }
	  else
	    {
#ifdef LIBOSC_DEBUG
	      cerr << "OSCProcessor::process(): no callable for address " << address << " found" << endl;
#endif

	      return false;
	    }
	}
      else
	{
#ifdef LIBOSC_DEBUG
	  cerr << "OSCProcessor::process(): no namespace set to this processor" << endl;
#endif
	}
    }

  return true;
}


void OSCProcessor::setNamespace (OSCNamespace *n)
{
  myNamespace = n;
}


OSCNamespace* OSCProcessor::getNamespace (void)
{
  return myNamespace;
}

// EOF
