/* Open SoundControl kit in C++                                              */
/* Copyright (C) 2002 Daniel Holth, Clinton McChesney                        */
/*                                                                           */
/* This library is free software; you can redistribute it and/or             */
/* modify it under the terms of the GNU Lesser General Public                */
/* License as published by the Free Software Foundation; either              */
/* version 2.1 of the License, or (at your option) any later version.        */
/*                                                                           */
/* This library is distributed in the hope that it will be useful,           */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of            */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         */
/* Lesser General Public License for more details.                           */
/*                                                                           */
/* You should have received a copy of the GNU Lesser General Public          */
/* License along with this library; if not, write to the Free Software       */
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
/*                                                                           */
/* For questions regarding this program contact                              */
/* Daniel Holth <dholth@stetson.edu> or visit                                */
/* http://www.stetson.edu/~ProctoLogic/                                      */

/**
 * @file QueuedBroadcaster.cc
 *
 *   
 *
 * @author Daniel Holth
 * @author Clinton McChesney
 */

#include "QueuedBroadcaster.h"
#include <sstream>
#include <pthread.h>

using namespace std;

QueuedBroadcaster::QueuedBroadcaster()
{
  pthread_mutex_init(this->getMutex(), NULL);
}


bool QueuedBroadcaster::transmitInternal(Transmit* destination, string *data)
{
  MessageQueue*           messages;
  sem_t*                  semaphore;
  pthread_mutex_t*        mutex;
  pair<Transmit*, string> item;

  messages    = this->getMessageQueue();
  semaphore   = this->getSemaphore();
  mutex       = this->getMutex();

  item = pair<Transmit*, string>(destination, *data);

  pthread_mutex_lock(mutex);
  messages->push(item);
  pthread_mutex_unlock(mutex);
  
  sem_post(semaphore);

  return true;
}


// must return true iff an element was removed from the queue
bool QueuedBroadcaster::sendOne()
{
  bool success;

  MessageQueue*           queue;
  pthread_mutex_t*        mutex;
  pair<Transmit*, string> item;

  success = false;
  queue   = this->getMessageQueue();
  mutex   = this->getMutex();

  pthread_mutex_lock(mutex);

  if(!queue->empty())
  {
    int length;
   
    // Get the item at the front of the queue.
    item   = queue->front();
    length = item.second.size();

#ifdef LIBOSC_DEBUG
    cout << "Length in QueuedBroadcaster: " << length << endl;
#endif
    
    // Remove it from the queue.
    queue->pop();
    
    // Send it.
    item.first->transmit(item.second);
    success = true;
  }
  else
  {
    success  = false;
  }

  pthread_mutex_unlock(mutex);

  return success;
}


bool QueuedBroadcaster::empty()
{
  /*MessageQueue* messages;
  bool empty;
  pthread_mutex_t* mutex;

  messages = this->getMessageQueue();
  
  
  pthread_mutex_lock(mutex);
  empty = messages->empty();
  pthread_mutex_unlock(mutex);

  return empty;*/
  return true;
}


std::string
QueuedBroadcaster::getName () const
{
  ostringstream name;
  name << "QueuedBroadcaster@" << this;
  return name.str();
}
